@@ -412,6 +412,9 @@ public:
     void updateXkbStateMask(const std::string &display, uint32_t depressed_mods,
                             uint32_t latched_mods, uint32_t locked_mods);
 
+    /// Clear xkb state mask for given display
+    void clearXkbStateMask(const std::string &display);
+
     /**
      * Show a small popup with input popup window with current input method
      * information.
@@ -467,6 +470,12 @@ public:
      */
     bool isRunning() const;
 
+    /**
+     * Whether restart is requested.
+     * @since 5.0.18
+     */
+    bool isRestartRequested() const;
+
 protected:
     // For testing purpose
     InstancePrivate *privateData();
