@@ -125,6 +125,7 @@ public:
     virtual ~StandardPathTempFile();
 
     int fd() const { return fd_.fd(); }
+    bool isValid() const { return fd_.isValid(); }
 
     const std::string &path() const { return path_; }
     const std::string &tempPath() const { return tempPath_; }
@@ -151,6 +152,7 @@ public:
     StandardPathFile &operator=(StandardPathFile &&other) = default;
 
     int fd() const { return fd_.fd(); }
+    bool isValid() const { return fd_.isValid(); }
 
     const std::string &path() const { return path_; }
 
@@ -171,8 +173,24 @@ typedef std::map<std::string, std::vector<StandardPathFile>>
 class FCITXUTILS_EXPORT StandardPath {
 public:
     /// \brief Enum for location type.
-    enum class Type { Config, PkgConfig, Data, Cache, Runtime, Addon, PkgData };
+    enum class Type {
+        /// Xdg Config dir
+        Config,
+        /// Xdg Config dir/fcitx5
+        PkgConfig,
+        /// Xdg data dir
+        Data,
+        /// Xdg cache dir
+        Cache,
+        /// Xdg runtime dir
+        Runtime,
+        /// addon shared library dir.
+        Addon,
+        /// Xdg data dir/fcitx5
+        PkgData
+    };
 
+    StandardPath(bool skipFcitxPath, bool skipUserPath);
     StandardPath(bool skipFcitxPath = false);
     virtual ~StandardPath();
 
@@ -200,6 +218,15 @@ public:
                          const std::function<bool(const std::string &path,
                                                   bool user)> &scanner) const;
 
+    /// \brief Scan the given directories.
+    ///
+    /// Callback returns true to continue the scan.
+    /// @since 5.0.4
+    void scanDirectories(
+        const std::string &userDir, const std::vector<std::string> &directories,
+        const std::function<bool(const std::string &path, bool user)> &scanner)
+        const;
+
     /// \brief Scan files scan file under [directory]/[path]
     /// \param path sub directory name.
     void scanFiles(Type type, const std::string &path,
@@ -231,14 +258,28 @@ public:
     StandardPathFile openUser(Type type, const std::string &path,
                               int flags) const;
 
+    /**
+     * \brief Open the non-user file.
+     *
+     * \since 5.0.6
+     */
+    StandardPathFile openSystem(Type type, const std::string &path,
+                                int flags) const;
+
     /// \brief Open user file, but create file with mktemp.
     StandardPathTempFile openUserTemp(Type type,
                                       const std::string &pathOrig) const;
 
-    /// \brief Save the file safely with write and rename to make sure the
-    /// operation is atomic.
-    /// \param callback Callback function that accept a file descriptor and
-    /// return whether the save if success or not.
+    /**
+     * \brief Save the file safely with write and rename to make sure the
+     * operation is atomic.
+     *
+     * Callback shall not close the file descriptor. If the API you are using
+     * does cannot do that, you may use UnixFD to help you dup it first.
+     *
+     * \param callback Callback function that accept a file descriptor and
+     * return whether the save if success or not.
+     */
     bool safeSave(Type type, const std::string &pathOrig,
                   const std::function<bool(int)> &callback) const;
 
@@ -279,6 +320,26 @@ public:
                                   filter::Chainer<Args...>(args...));
     }
 
+    int64_t timestamp(Type type, const std::string &path) const;
+
+    /**
+     * Check if certain executable presents in PATH.
+     *
+     * If the file presents, return the absolute PATH. If name is absolute path,
+     * check the absolute path instead of using PATH.
+     *
+     * @since 5.0.18
+     */
+    static std::string findExecutable(const std::string &name);
+
+    /**
+     * Check if certain executable presents in PATH.
+     *
+     * @see findExecutable     *
+     * @since 5.0.18
+     */
+    static bool hasExecutable(const std::string &name);
+
 private:
     std::unique_ptr<StandardPathPrivate> d_ptr;
     FCITX_DECLARE_PRIVATE(StandardPath);
